
/*******************************************************************************

 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

/*
 * This class is generated using freemarker and the AggrBitwiseLogicalTypeFunctions.java template.
 */

package org.apache.drill.exec.expr.fn.impl.gaggr;

import org.apache.drill.exec.expr.DrillAggFunc;
import org.apache.drill.exec.expr.annotations.FunctionTemplate;
import org.apache.drill.exec.expr.annotations.FunctionTemplate.NullHandling;
import org.apache.drill.exec.expr.annotations.FunctionTemplate.FunctionScope;
import org.apache.drill.exec.expr.annotations.Output;
import org.apache.drill.exec.expr.annotations.Param;
import org.apache.drill.exec.expr.annotations.Workspace;
import org.apache.drill.exec.expr.holders.BigIntHolder;
import org.apache.drill.exec.expr.holders.NullableBigIntHolder;
import org.apache.drill.exec.expr.holders.IntHolder;
import org.apache.drill.exec.expr.holders.NullableIntHolder;
import org.apache.drill.exec.expr.holders.SmallIntHolder;
import org.apache.drill.exec.expr.holders.NullableSmallIntHolder;
import org.apache.drill.exec.expr.holders.TinyIntHolder;
import org.apache.drill.exec.expr.holders.NullableTinyIntHolder;
import org.apache.drill.exec.expr.holders.UInt1Holder;
import org.apache.drill.exec.expr.holders.NullableUInt1Holder;
import org.apache.drill.exec.expr.holders.UInt2Holder;
import org.apache.drill.exec.expr.holders.NullableUInt2Holder;
import org.apache.drill.exec.expr.holders.UInt4Holder;
import org.apache.drill.exec.expr.holders.NullableUInt4Holder;
import org.apache.drill.exec.expr.holders.UInt8Holder;
import org.apache.drill.exec.expr.holders.NullableUInt8Holder;

@SuppressWarnings("unused")

public class BitwiseAndFunctions {
  static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(BitwiseAndFunctions.class);


@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class BigIntBitwiseAnd implements DrillAggFunc{

  @Param BigIntHolder in;
  @Workspace NullableBigIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableBigIntHolder out;

  public void setup() {
    inter = new NullableBigIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Integer.MAX_VALUE;
  }

  @Override
  public void add() {
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Integer.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class NullableBigIntBitwiseAnd implements DrillAggFunc{

  @Param NullableBigIntHolder in;
  @Workspace NullableBigIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableBigIntHolder out;

  public void setup() {
    inter = new NullableBigIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Integer.MAX_VALUE;
  }

  @Override
  public void add() {
    sout: {
    if (in.isSet == 0) {
     // processing nullable input and the value is null, so don't do anything...
     break sout;
    }
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

    } // end of sout block
  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Integer.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class IntBitwiseAnd implements DrillAggFunc{

  @Param IntHolder in;
  @Workspace NullableIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableIntHolder out;

  public void setup() {
    inter = new NullableIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Integer.MAX_VALUE;
  }

  @Override
  public void add() {
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Integer.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class NullableIntBitwiseAnd implements DrillAggFunc{

  @Param NullableIntHolder in;
  @Workspace NullableIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableIntHolder out;

  public void setup() {
    inter = new NullableIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Integer.MAX_VALUE;
  }

  @Override
  public void add() {
    sout: {
    if (in.isSet == 0) {
     // processing nullable input and the value is null, so don't do anything...
     break sout;
    }
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

    } // end of sout block
  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Integer.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class SmallIntBitwiseAnd implements DrillAggFunc{

  @Param SmallIntHolder in;
  @Workspace NullableSmallIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableSmallIntHolder out;

  public void setup() {
    inter = new NullableSmallIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Byte.MAX_VALUE;
  }

  @Override
  public void add() {
    nonNullCount.value = 1;
    inter.value = (short)(inter.value & in.value);

  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Byte.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class NullableSmallIntBitwiseAnd implements DrillAggFunc{

  @Param NullableSmallIntHolder in;
  @Workspace NullableSmallIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableSmallIntHolder out;

  public void setup() {
    inter = new NullableSmallIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Byte.MAX_VALUE;
  }

  @Override
  public void add() {
    sout: {
    if (in.isSet == 0) {
     // processing nullable input and the value is null, so don't do anything...
     break sout;
    }
    nonNullCount.value = 1;
    inter.value = (short)(inter.value & in.value);

    } // end of sout block
  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Byte.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class TinyIntBitwiseAnd implements DrillAggFunc{

  @Param TinyIntHolder in;
  @Workspace NullableTinyIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableTinyIntHolder out;

  public void setup() {
    inter = new NullableTinyIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Byte.MAX_VALUE;
  }

  @Override
  public void add() {
    nonNullCount.value = 1;
    inter.value = (byte)(inter.value & in.value);

  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Byte.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class NullableTinyIntBitwiseAnd implements DrillAggFunc{

  @Param NullableTinyIntHolder in;
  @Workspace NullableTinyIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableTinyIntHolder out;

  public void setup() {
    inter = new NullableTinyIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Byte.MAX_VALUE;
  }

  @Override
  public void add() {
    sout: {
    if (in.isSet == 0) {
     // processing nullable input and the value is null, so don't do anything...
     break sout;
    }
    nonNullCount.value = 1;
    inter.value = (byte)(inter.value & in.value);

    } // end of sout block
  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Byte.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class UInt1BitwiseAnd implements DrillAggFunc{

  @Param UInt1Holder in;
  @Workspace NullableIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableIntHolder out;

  public void setup() {
    inter = new NullableIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Byte.MAX_VALUE;
  }

  @Override
  public void add() {
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Byte.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class NullableUInt1BitwiseAnd implements DrillAggFunc{

  @Param NullableUInt1Holder in;
  @Workspace NullableIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableIntHolder out;

  public void setup() {
    inter = new NullableIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Byte.MAX_VALUE;
  }

  @Override
  public void add() {
    sout: {
    if (in.isSet == 0) {
     // processing nullable input and the value is null, so don't do anything...
     break sout;
    }
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

    } // end of sout block
  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Byte.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class UInt2BitwiseAnd implements DrillAggFunc{

  @Param UInt2Holder in;
  @Workspace NullableIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableIntHolder out;

  public void setup() {
    inter = new NullableIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Character.MAX_VALUE;
  }

  @Override
  public void add() {
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Character.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class NullableUInt2BitwiseAnd implements DrillAggFunc{

  @Param NullableUInt2Holder in;
  @Workspace NullableIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableIntHolder out;

  public void setup() {
    inter = new NullableIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Character.MAX_VALUE;
  }

  @Override
  public void add() {
    sout: {
    if (in.isSet == 0) {
     // processing nullable input and the value is null, so don't do anything...
     break sout;
    }
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

    } // end of sout block
  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Character.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class UInt4BitwiseAnd implements DrillAggFunc{

  @Param UInt4Holder in;
  @Workspace NullableIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableIntHolder out;

  public void setup() {
    inter = new NullableIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Integer.MAX_VALUE;
  }

  @Override
  public void add() {
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Integer.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class NullableUInt4BitwiseAnd implements DrillAggFunc{

  @Param NullableUInt4Holder in;
  @Workspace NullableIntHolder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableIntHolder out;

  public void setup() {
    inter = new NullableIntHolder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Integer.MAX_VALUE;
  }

  @Override
  public void add() {
    sout: {
    if (in.isSet == 0) {
     // processing nullable input and the value is null, so don't do anything...
     break sout;
    }
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

    } // end of sout block
  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Integer.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class UInt8BitwiseAnd implements DrillAggFunc{

  @Param UInt8Holder in;
  @Workspace NullableUInt8Holder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableUInt8Holder out;

  public void setup() {
    inter = new NullableUInt8Holder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Long.MAX_VALUE;
  }

  @Override
  public void add() {
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Long.MAX_VALUE;
  }
}



@FunctionTemplate(names = {"bit_and", "every"}, scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
public static class NullableUInt8BitwiseAnd implements DrillAggFunc{

  @Param NullableUInt8Holder in;
  @Workspace NullableUInt8Holder inter;
  @Workspace BigIntHolder nonNullCount;
  @Output NullableUInt8Holder out;

  public void setup() {
    inter = new NullableUInt8Holder();
    nonNullCount = new BigIntHolder();
    nonNullCount.value = 0;

    // Initialize the workspace variables
    inter.value = Long.MAX_VALUE;
  }

  @Override
  public void add() {
    sout: {
    if (in.isSet == 0) {
     // processing nullable input and the value is null, so don't do anything...
     break sout;
    }
    nonNullCount.value = 1;
    inter.value = (inter.value & in.value);

    } // end of sout block
  }

  @Override
  public void output() {
    if (nonNullCount.value > 0) {
      out.isSet = 1;
        out.value = inter.value;
    } else {
      out.isSet = 0;
    }
  }

  @Override
  public void reset() {
    nonNullCount.value = 0;
      inter.value = Long.MAX_VALUE;
  }
}


}
