
/*******************************************************************************

 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.drill.exec.expr.fn.impl.gcast;


import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkState;

import com.google.common.collect.Lists;
import com.google.common.collect.ObjectArrays;
import com.google.common.base.Charsets;
import com.google.common.collect.ObjectArrays;

import com.google.common.base.Preconditions;
import io.netty.buffer.*;

import org.apache.commons.lang3.ArrayUtils;

import org.apache.drill.common.exceptions.UserException;
import org.apache.drill.exec.expr.fn.impl.StringFunctionUtil;
import org.apache.drill.exec.memory.*;
import org.apache.drill.exec.proto.SchemaDefProtos;
import org.apache.drill.exec.proto.UserBitShared;
import org.apache.drill.exec.proto.UserBitShared.DrillPBError;
import org.apache.drill.exec.proto.UserBitShared.SerializedField;
import org.apache.drill.exec.record.*;
import org.apache.drill.exec.vector.*;
import org.apache.drill.common.exceptions.*;
import org.apache.drill.exec.exception.*;
import org.apache.drill.exec.expr.holders.*;
import org.apache.drill.common.expression.FieldReference;
import org.apache.drill.common.types.TypeProtos.*;
import org.apache.drill.common.types.Types;
import org.apache.drill.common.util.DrillStringUtils;
import org.apache.drill.exec.vector.complex.*;
import org.apache.drill.exec.vector.complex.reader.*;
import org.apache.drill.exec.vector.complex.impl.*;
import org.apache.drill.exec.vector.complex.writer.*;
import org.apache.drill.exec.vector.complex.writer.BaseWriter.MapWriter;
import org.apache.drill.exec.vector.complex.writer.BaseWriter.ListWriter;
import org.apache.drill.exec.util.JsonStringArrayList;

import org.apache.drill.exec.exception.OutOfMemoryException;

import com.sun.codemodel.JType;
import com.sun.codemodel.JCodeModel;

import javax.inject.Inject;

import java.util.Arrays;
import java.util.Random;
import java.util.List;

import java.io.Closeable;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;

import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.math.BigDecimal;
import java.math.BigInteger;

import org.joda.time.DateTime;
import org.joda.time.Period;

import org.apache.drill.exec.util.Text;

import org.apache.drill.exec.vector.accessor.sql.TimePrintMillis;
import javax.inject.Inject;






import org.apache.drill.exec.expr.DrillSimpleFunc;
import org.apache.drill.exec.expr.annotations.FunctionTemplate;
import org.apache.drill.exec.expr.annotations.FunctionTemplate.NullHandling;
import org.apache.drill.exec.expr.annotations.Output;
import org.apache.drill.exec.expr.annotations.Param;
import org.apache.drill.exec.expr.holders.*;
import org.apache.drill.exec.record.RecordBatch;
import org.apache.drill.exec.util.DecimalUtility;
import org.apache.drill.exec.expr.annotations.Workspace;

import io.netty.buffer.ByteBuf;

import java.nio.ByteBuffer;

/*
 * This class is generated using freemarker and the Decimal/CastDecimalVarchar.java template.
 */

@SuppressWarnings("unused")
@FunctionTemplate(name = "castVARCHAR", scope = FunctionTemplate.FunctionScope.SIMPLE, nulls=NullHandling.NULL_IF_NULL)
public class CastDecimal38SparseVarChar implements DrillSimpleFunc {

    @Param Decimal38SparseHolder in;
    @Param BigIntHolder len;
    @Inject DrillBuf buffer;
    @Output VarCharHolder out;

    public void setup() {
        buffer = buffer.reallocIfNeeded((int) len.value);
    }

    public void eval() {

        StringBuilder str = new StringBuilder();
        int index = 0;
        int fractionalStartIndex = 6 - org.apache.drill.exec.util.DecimalUtility.roundUp(in.scale);

        // Find the first non-zero value in the integer part of the decimal
        while (index < fractionalStartIndex && in.getInteger(index, in.start, in.buffer) == 0)  {
            index++;
        }


        // If we have valid digits print '-' sign
        if ((in.getSign(in.start, in.buffer) == true) && index < 6) {
            str.append("-");
        }

        // If all the integer digits are zero, print a single zero
        if (index == fractionalStartIndex) {
            str.append("0");
        }

        boolean fillZeroes = false;

        // convert the integer part
        while (index < fractionalStartIndex) {
            int value =  in.getInteger(index++, in.start, in.buffer);

            if (fillZeroes == true) {
                str.append(org.apache.drill.exec.util.DecimalUtility.toStringWithZeroes(value, org.apache.drill.exec.util.DecimalUtility.MAX_DIGITS));
            } else {
                str.append(value);
                fillZeroes = true;
            }
            //str.append(value);
        }

        if (fractionalStartIndex < 6) {
            // We have fractional part, print '.'
            str.append(".");

            /* convert the fractional part (except the last decimal digit,
             * as it might have padding that needs to be stripped
             */
            while (fractionalStartIndex < 6 - 1) {
                int value = in.getInteger(fractionalStartIndex++, in.start, in.buffer);

                // Fill zeroes at the beginning of the decimal digit
                str.append(org.apache.drill.exec.util.DecimalUtility.toStringWithZeroes(value, org.apache.drill.exec.util.DecimalUtility.MAX_DIGITS));
            }

            // Last decimal digit, strip the extra zeroes we may have padded
            int actualDigits = in.scale % org.apache.drill.exec.util.DecimalUtility.MAX_DIGITS;

            int lastFractionalDigit = in.getInteger(6 - 1, in.start, in.buffer);

            if (actualDigits != 0) {

                // Strip padded zeroes at the end that is not part of the scale
                lastFractionalDigit /= (int) (org.apache.drill.exec.util.DecimalUtility.getPowerOfTen((int) (org.apache.drill.exec.util.DecimalUtility.MAX_DIGITS - actualDigits)));
                str.append(org.apache.drill.exec.util.DecimalUtility.toStringWithZeroes(lastFractionalDigit, actualDigits));
            } else {
                // Last digit does not have any padding print as is
                str.append(org.apache.drill.exec.util.DecimalUtility.toStringWithZeroes(lastFractionalDigit, org.apache.drill.exec.util.DecimalUtility.MAX_DIGITS));
            }


        }

        out.buffer = buffer;
        out.start = 0;
        out.end = Math.min((int)len.value, str.length()); // truncate if target type has length smaller than that of input's string
        out.buffer.setBytes(0, String.valueOf(str.substring(0,out.end)).getBytes());
    }
}
 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 

 
